@@ -604,28 +604,29 @@ TYPE specifies the archiver's symbol."
       (setq newfile (elmo-concat-path
 		     prefix
 		     (int-to-string next-num)))
-      (unwind-protect
-	  (elmo-bind-directory
-	   tmp-dir
-	   (if (and (or (functionp method) (car method))
-		    (file-writable-p newfile))
-	       (progn
-		 (setq dst-buffer (current-buffer))
-		 (with-current-buffer src-buffer
-		   (copy-to-buffer dst-buffer (point-min) (point-max)))
-		 (as-binary-output-file
+      (elmo-bind-directory
+       tmp-dir
+       (if (and (or (functionp method) (car method))
+		(file-writable-p newfile))
+	   (progn
+	     (setq dst-buffer (current-buffer))
+	     (with-current-buffer src-buffer
+	       (copy-to-buffer dst-buffer (point-min) (point-max)))
+	     (as-binary-output-file
 		  (write-region (point-min) (point-max) newfile nil 'no-msg))
-		 (when (elmo-archive-call-method method (list arc newfile))
-		   (elmo-folder-preserve-flags
-		    folder
-		    (with-current-buffer src-buffer
-		      (elmo-msgdb-get-message-id-from-buffer))
-		    flags)
-		   t))
-	     nil))))))
+	     (when (elmo-archive-call-method method (list arc newfile))
+	       (elmo-folder-preserve-flags
+		folder
+		(with-current-buffer src-buffer
+		  (elmo-msgdb-get-message-id-from-buffer))
+		flags)
+	       t))
+	 nil)))))
 
-(luna-define-method elmo-folder-append-messages :around
-  ((folder elmo-archive-folder) src-folder numbers &optional same-number)
+(defun elmo-folder-append-messages-*-archive (folder
+					      src-folder
+					      numbers
+					      same-number)
   (let ((prefix (elmo-archive-folder-archive-prefix-internal folder)))
     (cond
      ((and same-number
@@ -633,9 +634,10 @@ TYPE specifies the archiver's symbol."
 	   (elmo-folder-message-file-p src-folder)
 	   (elmo-folder-message-file-number-p src-folder))
       ;; same-number(localdir, localnews) -> archive
-      (unless (elmo-archive-append-files folder
-					 (elmo-folder-message-file-directory src-folder)
-					 numbers)
+      (unless (elmo-archive-append-files
+	       folder
+	       (elmo-folder-message-file-directory src-folder)
+	       numbers)
 	(setq numbers nil))
       (elmo-progress-notify 'elmo-folder-move-messages (length numbers))
       numbers)
@@ -649,33 +651,37 @@ TYPE specifies the archiver's symbol."
 						 folder))
 				 (car (elmo-folder-status folder)) 0)))))
 	    new-dir base-dir files)
-	(setq base-dir temp-dir)
-	(when (> (length prefix) 0)
-	  (when (file-name-directory prefix)
-	    (elmo-make-directory (file-name-directory prefix)))
-	  (rename-file
-	   temp-dir
-	   (setq new-dir
-		 (expand-file-name
-		  prefix
-		  ;; parent of temp-dir..(works in windows?)
-		  (expand-file-name ".." temp-dir))))
-	  ;; now temp-dir has name prefix.
-	  (setq temp-dir new-dir)
-	  ;; parent of prefix becomes base-dir.
-	  (setq base-dir (expand-file-name ".." temp-dir)))
-	(setq files
-	      (mapcar
-	       '(lambda (x) (elmo-concat-path prefix x))
-	       (directory-files temp-dir nil "^[^\\.]")))
-	(if (elmo-archive-append-files folder
-				       base-dir
-				       files)
-	    (elmo-delete-directory temp-dir)
-	  (setq numbers nil)))
+	(unwind-protect
+	    (progn
+	      (setq base-dir temp-dir)
+	      (when (> (length prefix) 0)
+		(when (file-name-directory prefix)
+		  (elmo-make-directory (file-name-directory prefix)))
+		(rename-file
+		 temp-dir
+		 (setq new-dir
+		       (expand-file-name
+			prefix
+			;; parent of temp-dir..(works in windows?)
+			(expand-file-name ".." temp-dir))))
+		;; now temp-dir has name prefix.
+		(setq temp-dir new-dir)
+		;; parent of prefix becomes base-dir.
+		(setq base-dir (expand-file-name ".." temp-dir)))
+	      (setq files
+		    (mapcar
+		     '(lambda (x) (elmo-concat-path prefix x))
+		     (directory-files temp-dir nil "^[^\\.]")))
+	      (unless (elmo-archive-append-files folder
+						 base-dir
+						 files)
+		(setq numbers nil)))
+	  (elmo-delete-directory temp-dir)))
       (elmo-progress-notify 'elmo-folder-move-messages (length numbers))
       numbers)
-     (t (luna-call-next-method)))))
+     (t
+      (elmo-folder-append-messages folder src-folder numbers same-number
+				   'elmo-folder-append-messages-*-archive)))))
 
 (luna-define-method elmo-folder-message-make-temp-file-p
   ((folder elmo-archive-folder))
