@@ -36,7 +36,8 @@
 (require 'elmo-map)
 
 (eval-and-compile
-  (luna-define-class elmo-cache-folder (elmo-map-folder) (dir-name directory))
+  (luna-define-class elmo-cache-folder (elmo-map-folder elmo-file-tag)
+		     (dir-name directory))
   (luna-define-internal-accessors 'elmo-cache-folder))
 
 (luna-define-method elmo-folder-initialize ((folder elmo-cache-folder)
@@ -135,12 +136,18 @@
   (dolist (location locations)
     (elmo-file-cache-delete
      (expand-file-name location
-		       (elmo-cache-folder-directory-internal folder)))))
+		       (elmo-cache-folder-directory-internal folder))))
+  t)
 
-(luna-define-method elmo-message-fetch-with-cache-process
-  ((folder elmo-cache-folder) number strategy &optional section unseen)
-  ;; disbable cache process
-  (elmo-message-fetch-internal folder number strategy section unseen))
+(luna-define-method elmo-message-fetch
+  ((folder elmo-cache-folder) number strategy &optional unseen section)
+  ;; disable cache process
+  (erase-buffer)
+  (when (elmo-message-fetch-internal folder number strategy section unseen)
+    (when (and (not unseen)
+	       (elmo-message-flagged-p folder number 'unread))
+      (elmo-message-unset-flag folder number 'unread))
+    t))
 
 (luna-define-method elmo-map-message-fetch ((folder elmo-cache-folder)
 					    location strategy
