@@ -84,11 +84,22 @@
   (or (elmo-folder-contains-type (elmo-pipe-folder-src-internal folder) type)
       (elmo-folder-contains-type (elmo-pipe-folder-dst-internal folder) type)))
 
-(luna-define-method elmo-folder-append-messages ((folder elmo-pipe-folder)
-						 src-folder numbers
-						 &optional same-number)
-  (elmo-folder-append-messages (elmo-pipe-folder-dst-internal folder)
-			       src-folder numbers
+(defun elmo-folder-append-messages-pipe-* (dst-folder
+					   src-folder
+					   numbers
+					   same-number)
+  (elmo-folder-append-messages dst-folder
+			       (elmo-pipe-folder-dst-internal src-folder)
+			       numbers
+			       same-number))
+
+(defun elmo-folder-append-messages-*-pipe (dst-folder
+					   src-folder
+					   numbers
+					   same-number)
+  (elmo-folder-append-messages (elmo-pipe-folder-dst-internal dst-folder)
+			       src-folder
+			       numbers
 			       same-number))
 
 (luna-define-method elmo-folder-append-buffer ((folder elmo-pipe-folder)
@@ -120,37 +131,30 @@
 (defvar elmo-pipe-drained-hook nil "A hook called when the pipe is flushed.")
 
 (defsubst elmo-pipe-folder-list-target-messages (src &optional ignore-list)
-  (unwind-protect
-      (progn
-	(elmo-folder-set-killed-list-internal src ignore-list)
-	(elmo-folder-list-messages src t))
-    (elmo-folder-set-killed-list-internal src nil)))
+  (let ((killed (elmo-folder-killed-list-internal src)))
+    (elmo-folder-set-killed-list-internal src ignore-list)
+    (unwind-protect
+	(elmo-folder-list-messages src t)
+      (elmo-folder-set-killed-list-internal src killed))))
 
 (defun elmo-pipe-drain (src dst &optional copy ignore-list)
   "Move or copy all messages of SRC to DST."
-  (let ((elmo-inhibit-number-mapping (and (eq (elmo-folder-type-internal
-					       src) 'pop3)
-					  (not copy))) ; No need to use UIDL
-	msgs len)
+  (let ((elmo-inhibit-number-mapping (and (eq (elmo-folder-type-internal src)
+					      'pop3)
+					  (not copy)))) ; No need to use UIDL
     (message "Checking %s..." (elmo-folder-name-internal src))
-    ;; Warnnig: some function requires msgdb
-    ;; but elmo-folder-open-internal do not load msgdb.
-    (elmo-folder-open-internal src)
-    (setq msgs (elmo-pipe-folder-list-target-messages src ignore-list)
-	  len (length msgs))
-    (when (> len elmo-display-progress-threshold)
-      (elmo-progress-set 'elmo-folder-move-messages
-			 len
-			 (if copy
-			     "Copying messages..."
-			   "Moving messages...")))
+    (elmo-folder-open src)
     (unwind-protect
-	(elmo-folder-move-messages src msgs dst copy)
-      (elmo-progress-clear 'elmo-folder-move-messages))
-    (when (and copy msgs)
-      (setq ignore-list (elmo-number-set-append-list ignore-list
-						     msgs)))
-    (elmo-folder-close-internal src)
+	(let* ((msgs (elmo-pipe-folder-list-target-messages src ignore-list))
+	       (len (length msgs)))
+	  (elmo-with-progress-display (> len elmo-display-progress-threshold)
+	      (elmo-folder-move-messages len (if copy
+						 "Copying messages..."
+					       "Moving messages..."))
+	    (elmo-folder-move-messages src msgs dst copy))
+	  (when (and copy msgs)
+	    (setq ignore-list (elmo-number-set-append-list ignore-list msgs))))
+      (elmo-folder-close src))
     (run-hooks 'elmo-pipe-drained-hook)
     ignore-list))
 
@@ -325,6 +329,8 @@
     (elmo-folder-rename (elmo-pipe-folder-dst-internal folder)
 			(elmo-folder-name-internal
 			 (elmo-pipe-folder-dst-internal new-folder)))
+    (elmo-global-flag-replace-referrer (elmo-folder-name-internal folder)
+				       new-name)
     (elmo-msgdb-rename-path folder new-folder)))
 
 (luna-define-method elmo-folder-synchronize ((folder elmo-pipe-folder)
