@@ -60,9 +60,7 @@
 (fset 'elmo-base64-decode-string
       (mel-find-function 'mime-decode-string "base64"))
 
-;; Any Emacsen may have add-name-to-file(), because loadup.el requires it. :-p
-;; Check make-symbolic-link() instead.  -- 981002 by Fuji
-(if (fboundp 'make-symbolic-link)  ;; xxx
+(if elmo-use-hardlink
     (defalias 'elmo-add-name-to-file 'add-name-to-file)
   (defun elmo-add-name-to-file
     (filename newname &optional ok-if-already-exists)
@@ -1306,11 +1304,11 @@ SPEC is a list as followed (LABEL MAX-VALUE [FORMAT])."
 
 (defun elmo-flatten (list-of-list)
   "Flatten LIST-OF-LIST."
-  (if list-of-list
-      (apply #'nconc
-	     (mapcar (lambda (element)
-		       (if (consp element) element (list element)))
-		     list-of-list))))
+  (and list-of-list
+       (apply #'append
+	      (mapcar (lambda (element)
+			(if (listp element) element (list element)))
+		      list-of-list))))
 
 (defun elmo-y-or-n-p (prompt &optional auto default)
   "Same as `y-or-n-p'.
